#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "bergenListener.h"
#include "bergenServer.h"
#include "bergenAmplitude.h"


// The constructor must be given the server object pointer. The Sound Object 
// itself is created by snerd on snerd's side.  Snerd is managing the data and 
// return just the Handle to it 
bergenAmplitude::bergenAmplitude(bergenServer *server) : bergenSound(server)
	{
	amplitude_ = 0;
	createRemote();
	}


// Process a message from the server
void bergenAmplitude::message(osc::ReceivedMessage::const_iterator arg)
	{
	const char* str = arg->AsString();
	if (!strcasecmp(str,"amplitude"))
		amplitude_ = (++arg)->AsFloat();
	else
		bergenSound::message(arg);
	}

// Sends message to the associated server to create a sound sample 
// and waits for the resulting handle to store
void bergenAmplitude::createRemote(void)
	{
	if (server_)
		{
		char buffer[1024];
		osc::OutboundPacketStream p( buffer, 1024 );
		p << osc::BeginMessage( "server" ) << "amplitude" << server_->listener()->handle() << osc::EndMessage;	
		server_->sendMessage( p );
		handle_ = server_->receiveHandle();
		}
	else
		handle_ = 0;
	}


// Returns the total amplitude of the input. If the server message failed,
// this will return 0. Note that this* function requires sending a message to 
// the server and waiting for the response, so it is potentially slow.
float bergenAmplitude::amplitude(void)
	{
	if (server_)
		{
		char buffer[1024];
		osc::OutboundPacketStream p( buffer, 1024 );
		p << osc::BeginMessage( "object" ) << handle_ << "getamplitude" << osc::EndMessage;	
		server_->sendMessage( p );
		}
	return amplitude_;
	}
