#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "osc/OscOutboundPacketStream.h"
#include "bergenServer.h"
#include "bergenSample.h"


// The constructor must be given the name of the sample file which the sound
// will play, in addition to the server object pointer. The Sound Object
// itself is created by snerd on snerd's side.Snerd is managing the data and
// return just the Handle to it 
bergenSample::bergenSample(char *filename,bergenServer *server) : bergenSound(server)
	{
	if (filename)
		{
		filename_ = strdup(filename);
		createRemote();
		}
	else
		{
		fprintf(stderr,"ERROR: bergenSample created with NULL file name\n");
		filename_ = NULL;
		}
	}


// Sends message to the associated server to create a sound sample 
// and waits for the resulting handle to store
void bergenSample::createRemote(void)
	{
	if (server_)
		{
		char buffer[1024];
		osc::OutboundPacketStream p( buffer, 1024 );
		p << osc::BeginMessage( "server" ) << "sample" << filename_ << server_->listener()->handle() << osc::EndMessage;
		server_->sendMessage( p );
		handle_ = server_->receiveHandle();
		}
	else
		handle_ = 0;
	}


// Sets the sound's looping flag -- if it is true (non-zero), the sound will
// loop continuously when it is played.
void bergenSample::setLoop(int loop)
	{
	if (server_)
		{
		char buffer[1024];
		osc::OutboundPacketStream p( buffer, 1024 );
		p << osc::BeginMessage( "object" ) << handle_ << "loop" << loop << osc::EndMessage;	
		server_->sendMessage( p );
		}
	}

